import glob
import os
from pathlib import Path


# todo: refactor
class Mindmap(object):
    def __init__(self, scope, edit_mode):
        self.edit_mode = edit_mode
        self.working_dir = f"{Path.home()}/mindmap/"
        self.files = self.__get_files_in_scope(scope)
        self.path = self.__get_full_scope_path(scope)
        print(f"working path: {self.path}")

    def __call__(self):
        pass

    # public
    def read_note(self):
        for file in self.files:
            self.read_index(self.files.index(file))

    def save_note(self, text):
        note = self.__format_note(text)
        note_file = open(self.files[0], "a+")
        note_file.writelines(note)

        print(f"note saved to {self.files[0]}")
        pass

    def list_scope(self):
        for file in self.files:
            self.__print_file(self.files.index(file), file)

    def query_scope(self, text):
        for file in self.files:
            if text in file:
                self.__print_file(self.files.index(file), file)

    def read_index(self, index):
        if index > len(self.files):
            print(f"missing index: {index}")
            return
        filename = self.files[index]
        self.__print_file(index, filename)
        print(open(filename).read())

    def __format_note(self, note):
        return f"{note}\n"

    def __print_file(self, index, file):
        file_info = f"{index}: {file}"
        print(file_info)

    # private
    def __get_full_scope_path(self, scope):
        return self.working_dir + "/".join(scope)

    def __get_files_in_scope(self, scope):
        scope_path = self.__get_full_scope_path(scope)
        if os.path.exists(scope_path):
            return [f for f in glob.glob(scope_path + "**/*.md", recursive=True)]
        elif os.path.exists(f"{scope_path}.md"):
            return [f"{scope_path}.md"]
        elif self.edit_mode:
            return [self.__create_folder(scope)]
        else:
            return []

    def __create_folder(self, scope):

        path = self.working_dir + "/".join(scope[:-1])
        Path(path).mkdir(parents=True, exist_ok=True)
        filepath = self.__get_full_scope_path(scope) + ".md"
        with open(filepath, 'w'):
            pass
        return filepath



import argparse
import subprocess
import tempfile

from mindmap import Mindmap

parser = argparse.ArgumentParser("Command line tool for keeping notes simple way.")


def parse_args():
    parser.add_argument('-s', '--scope', type=str, nargs='+', help='scope of the note.', default=["general", "todo"])
    parser.add_argument('-q', '--query', type=str, help='search file names in scope.')
    parser.add_argument('-i', '--index', type=int, help='use index of the file.')

    parser.add_argument('-t', '--text', type=str, help='append text to scope content.')

    parser.add_argument('-a', '--all', action='store_true', help='set scope to all')
    parser.add_argument('-v', '--vim', action='store_true', help='save the content of vim')

    parser.add_argument('-r', '--read', action='store_true', help='print content of the scope.')
    parser.add_argument('-l', '--list', action='store_true', help='list files in scope.')
    parser.add_argument('-f', '--full_path', action='store_true', help='print with full path', default=False)

    return parser.parse_args()


def read_note_in_vim():
    with tempfile.NamedTemporaryFile(suffix='task') as temp:
        subprocess.call(['vim', temp.name])
        return open(temp.name, 'r').read()


if __name__ == "__main__":
    args = parse_args()

    scope = [] if args.all else args.scope
    edit_mode = args.text or args.vim
    t = Mindmap(scope, edit_mode)
    if args.list:
        t.list_scope()
    elif args.query:
        t.query_scope(args.query)
    elif args.read:
        t.read_note()
    elif args.index:
        t.read_index(args.index)
    elif args.text:
        t.save_note(args.text)
    elif args.vim:
        text = read_note_in_vim()
        t.save_note(text)
    else:
        parser.print_help()
